// Copyright (c) 2019-2020 The Authors
// This file is part of @npcz/ngx-highlight
// https://github.com/abdes/happy-coding
//
// SPDX-License-Identifier: BSD-3-Clause
import { __assign, __decorate, __metadata } from "tslib";
import { Directive, ElementRef, HostListener, Input, Renderer2, OnDestroy, } from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { debounceTime } from 'rxjs/operators';
/**
 * Default highlight styles: `none` for hover and `none` for focus.
 */
export var DEFAULT_STYLE_CONFIG = {
    hover: 'none',
    focus: 'none',
};
/**
 * Default debounce time for mouse/focus events.
 */
export var DEFAULT_DEBOUNCE_TIME = 50;
var HighlightDirective = /** @class */ (function () {
    function HighlightDirective(_renderer, _el) {
        this._renderer = _renderer;
        this._el = _el;
        this._style = __assign({}, DEFAULT_STYLE_CONFIG);
        this._debounceTime = DEFAULT_DEBOUNCE_TIME;
        this._hovered$ = new BehaviorSubject(false);
        this._isHovered = false;
        this._focused$ = new BehaviorSubject(false);
        this._isFocused = false;
        this._subscriptions = {
            hover: undefined,
            focus: undefined,
        };
    }
    Object.defineProperty(HighlightDirective.prototype, "highlightStyle", {
        /**
         * The currently set highlight styles for the directive.
         */
        get: function () {
            return this._style;
        },
        set: function (value) {
            this._updateStyleConfig(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighlightDirective.prototype, "hcHighlightDebounce", {
        /**
         * The currently set debounce time for the directive.
         *
         * Events are debounced to limit the frequency at which hover and/or focus
         * styles are applied/removed and provide a nicer user experience. A value of
         * `0` completely disables debouncing.
         */
        get: function () {
            return this._debounceTime;
        },
        set: function (value) {
            this._updateDebounceTime(value);
        },
        enumerable: true,
        configurable: true
    });
    HighlightDirective.prototype.ngOnDestroy = function () {
        this._stopFocusSubscription();
        this._stopHoverSubscription();
    };
    HighlightDirective.prototype.onMouseEnter = function () {
        this._hovered$.next(true);
    };
    HighlightDirective.prototype.onMouseLeave = function () {
        this._hovered$.next(false);
    };
    HighlightDirective.prototype.onFocusIn = function () {
        this._focused$.next(true);
    };
    HighlightDirective.prototype.onFocusOut = function () {
        this._focused$.next(false);
    };
    HighlightDirective.prototype._updateStyleConfig = function (value) {
        // Merge the provided config with the default config
        var newStyle = __assign({}, DEFAULT_STYLE_CONFIG);
        value &&
            (newStyle = __assign(__assign({}, newStyle), value));
        // Remove the old effects if the new ones are different,
        // they will be applied again with the new config as appropriate
        if (newStyle.hover != this._style.hover)
            this._removeEffect('hover');
        if (newStyle.focus != this._style.focus)
            this._removeEffect('focus');
        // Switch the style config to the new one and restart the subscriptions as
        // appropriate
        this._style = newStyle;
        // Hover
        if (this._style.hover != 'none') {
            this._startHoverSubscription();
        }
        else {
            // otherwise, just reset the current hover state
            this._isHovered = false;
        }
        // Focus
        if (this._style.focus != 'none') {
            this._startFocusSubscription();
        }
        else {
            // otherwise, just reset the current focus state
            this._isFocused = false;
        }
    };
    HighlightDirective.prototype._updateDebounceTime = function (value) {
        // Adjust debounceTime if it is negative
        var adjustedDebounceTime = DEFAULT_DEBOUNCE_TIME;
        if (value) {
            adjustedDebounceTime = value > 0 ? value : 0;
        }
        if (this._debounceTime != adjustedDebounceTime) {
            this._debounceTime = adjustedDebounceTime;
            if (this._subscriptions.hover) {
                this._stopHoverSubscription();
                this._startHoverSubscription();
            }
            if (this._subscriptions.focus) {
                this._stopFocusSubscription();
                this._startFocusSubscription();
            }
        }
    };
    HighlightDirective.prototype._startHoverSubscription = function () {
        var _this = this;
        this._subscriptions.hover = this._hovered$
            .pipe(debounceTime(this._debounceTime))
            .subscribe(function (status) {
            _this._isHovered = status;
            // Do not apply the hover effect if the element is focused
            if (_this._isHovered && !_this._isFocused)
                _this._applyEffect('hover');
            else
                _this._removeEffect('hover');
        });
        this._hovered$.next(this._isHovered);
    };
    HighlightDirective.prototype._stopHoverSubscription = function () {
        var _a;
        (_a = this._subscriptions.hover) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this._subscriptions.hover = undefined;
    };
    HighlightDirective.prototype._startFocusSubscription = function () {
        var _this = this;
        this._subscriptions.focus = this._focused$
            .pipe(debounceTime(this._debounceTime))
            .subscribe(function (status) {
            _this._isFocused = status;
            if (_this._isFocused) {
                _this._applyEffect('focus');
                // If the element is hovered, then remove the hover effect
                if (_this._isHovered)
                    _this._removeEffect('hover');
            }
            else {
                _this._removeEffect('focus');
                // If the element is hovered, then apply the hover effect
                if (_this._isHovered)
                    _this._applyEffect('hover');
            }
        });
        this._focused$.next(this._isFocused);
    };
    HighlightDirective.prototype._stopFocusSubscription = function () {
        var _a;
        (_a = this._subscriptions.focus) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this._subscriptions.focus = undefined;
    };
    HighlightDirective.prototype._applyEffect = function (effect) {
        var style = this._style[effect];
        console.assert(style != 'none');
        this._renderer.addClass(this._el.nativeElement, "hc-" + effect + "-" + style);
    };
    HighlightDirective.prototype._removeEffect = function (effect) {
        var style = this._style[effect];
        if (style != 'none') {
            this._renderer.removeClass(this._el.nativeElement, "hc-" + effect + "-" + style);
        }
    };
    HighlightDirective.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ElementRef }
    ]; };
    __decorate([
        Input('hcHighlight'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], HighlightDirective.prototype, "highlightStyle", null);
    __decorate([
        Input(),
        __metadata("design:type", Number),
        __metadata("design:paramtypes", [Number])
    ], HighlightDirective.prototype, "hcHighlightDebounce", null);
    __decorate([
        HostListener('mouseenter'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onMouseEnter", null);
    __decorate([
        HostListener('mouseleave'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onMouseLeave", null);
    __decorate([
        HostListener('focusin'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onFocusIn", null);
    __decorate([
        HostListener('focusout'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onFocusOut", null);
    HighlightDirective = __decorate([
        Directive({
            selector: '[hcHighlight]',
        }),
        __metadata("design:paramtypes", [Renderer2, ElementRef])
    ], HighlightDirective);
    return HighlightDirective;
}());
export { HighlightDirective };
//# sourceMappingURL=highlight.directive.js.map